class JobDisplay extends RessourceDisplay {
  constructor(displayElement) {
    super(displayElement);
    this.jobId = this.displayElement.dataset.jobId;
  }

  init(user) {
    let job = user.jobs[this.jobId];
    this.setCreationDate(job.creation_date);
    this.setEndDate(job.creation_date);
    this.setDescription(job.description);
    this.setService(job.service);
    this.setServiceArgs(job.service_args);
    this.setServiceVersion(job.service_version);
    this.setStatus(job.status);
    this.setTitle(job.title);
  }

  usersPatchHandler(patch) {
    let re = new RegExp(`^/users/${this.userId}/jobs/${this.jobId}`);
    let filteredPatch = patch.filter(operation => re.test(operation.path));
    for (let operation of filteredPatch) {
      switch(operation.op) {
        case 'replace':
          re = new RegExp(`^/users/${this.userId}/jobs/${this.jobId}/end_date$`);
          if (re.test(operation.path)) {
            this.setEndDate(operation.value);
            break;
          }
          re = new RegExp(`^/users/${this.userId}/jobs/${this.jobId}/status$`);
          if (re.test(operation.path)) {
            this.setStatus(operation.value);
            break;
          }
          break;
        default:
          break;
      }
    }
  }

  setTitle(title) {
    for (let element of this.displayElement.querySelectorAll('.job-title')) {this.setElement(element, title);}
  }

  setDescription(description) {
    for (let element of this.displayElement.querySelectorAll('.job-description')) {this.setElement(element, description);}
  }

  setStatus(status) {
    for (let element of this.displayElement.querySelectorAll('.job-status')) {
      this.setElement(element, status);
    }
    for (let element of this.displayElement.querySelectorAll('.status')) {element.dataset.status = status;}
    for (let element of this.displayElement.querySelectorAll('.status-spinner')) {
      if (['complete', 'failed'].includes(status)) {
        element.classList.add('hide');
      } else {
        element.classList.remove('hide');
      }
    }
    for (let element of this.displayElement.querySelectorAll('.restart-job-trigger')) {
      if (['complete', 'failed'].includes(status)) {
        element.classList.remove('hide');
      } else {
        element.classList.add('hide');
      }
    }
  }

  setCreationDate(creationDate) {
    for (let element of this.displayElement.querySelectorAll('.job-creation-date')) {
      this.setElement(element, creationDate.toLocaleString('en-US'));
    }
  }

  setEndDate(endDate) {
    for (let element of this.displayElement.querySelectorAll('.job-end-date')) {
      this.setElement(element, endDate.toLocaleString('en-US'));
    }
  }

  setService(service) {
    for (let element of this.displayElement.querySelectorAll('.job-service')) {this.setElement(element, service);}
  }

  setServiceArgs(serviceArgs) {
    for (let element of this.displayElement.querySelectorAll('.job-service-args')) {this.setElement(element, serviceArgs);}
  }

  setServiceVersion(serviceVersion) {
    for (let element of this.displayElement.querySelectorAll('.job-service-version')) {this.setElement(element, serviceVersion);}
  }
}
